(:~
 : This module creates the single edition view. According to the GET parameter
 : "edition" this module gets from edition.xqm all common text lines and the ones
 : that fit a tei:rdg are selected.
 :
 : The whole module is called by xml-utils.xqm.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 :)

xquery version "3.1";

module namespace single="http://bdn-edition.de/ns/single";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html5="http://www.w3.org/1999/xhtml";

import module namespace commons="http://bdn-edition.de/ns/commons" at "commons.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx = "http://www.functx.com";
import module namespace id-file="https://bdn-edition.de/ns/id-file" at "id-file.xqm";

declare variable $single:id-map :=
  map:merge
  (
    for $id in doc($config:aux || "id-file.xml")//bdn:id
        return map:entry($id/string(), $id/parent::bdn:file/@uri/string())
  );
(: Wozu wird die Variable $single:id-map genutzt?! :)

(:~
 : This function calls the witness-element according to the current edition.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 :)

declare function single:witness( $node, $mode ){
    $node/root()//tei:witness[@xml:id = $mode]
};

(:~
 : This function calls the rend-attribute of the witness-element according to the current edition.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 :)

declare function single:wit-rend( $node, $mode ){
    data(single:witness($node, $mode)/@rend)
};

(:~
 : This function creates a q with a certain class, inside quotation marks at the end and the beginning. This function calls the functions single:apply-corresp-node, single:apply-join and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $markbegin: quotation mark at the beginning
 : @param $markend: quoatation mark at the end
 :)

declare function single:qqmclass( $node, $mode, $file, $markbegin as xs:string, $markend as xs:string, $class as xs:string ){
    element html5:q {
        attribute class { $class },
        $markbegin,
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
            single:apply-join($node, $mode, $file)
        else
            single:transform($node/node(), $mode, $file),
        $markend
    }  
};

(:~
 : This function creates a div with a certain class and calls the functions single:apply-corresp-node, single:apply-join and single:transform.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:divclass
    ( $node, $mode, $file, $class as xs:string ){
    element html5:div {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
            single:apply-join($node, $mode, $file)
        else
            single:transform($node/node(), $mode, $file)
    }
};

(:~
 : This function uses the function single:divclass to render blocklevel-elements like tei:p and tei:lg.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $blockelement: Defining the kind of blockelement (e.g. "p" or "line")
 :)

declare function single:blockelement( $node, $mode, $file, $blockelement as xs:string ) {
    if ($node/@rend= "blockquote") then
        single:divclass($node, $mode, $file, "blockquote " || $blockelement)
    else if ($node/@rend= "blockquoteitalics") then
        single:divclass($node, $mode, $file, "italics " || $blockelement)
    else if ($node/@rend = "small_letters") then
        single:divclass($node, $mode, $file, "small-letters " || $blockelement)
    else
       single:divclass($node, $mode, $file, $blockelement)
};

(:~
 : This function creates a span with a certain class and calls the functions single:apply-corresp-node, single:apply-join and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:spanclass
    ( $node, $mode, $file, $class as xs:string ){
    element html5:span {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
            single:apply-join($node, $mode, $file)
        else
            single:transform($node/node(), $mode, $file)
    }
};

 (:~
 : This function is used for additions in the text. It creates a span with a certain class and a tooltip for showing the responsibility. It the functions single:apply-corresp-node and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:spanclasstooltip
    ( $node, $mode, $file, $class as xs:string ){
    element html5:span {
        attribute class { $class || " resp-tt" },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
            single:apply-join($node, $mode, $file)
        else
            single:transform($node/node(), $mode, $file),
        element html5:span
        {
            attribute class {"resp-tttext"},
            "Eingefügt von ",
            normalize-space(data(single:linkelement($node))) || " ",
            if (single:fullink( $node )) then
                single:createPopover(
                     "fas fa-user",
                    'kmg-pnd',
                    'Angaben aus der GND',
                    'Laden ...',
                  normalize-space(replace(data(single:fullink($node)), "gnd:", "")),
                    ''
                )
            else
                ()
        }
    }
};

(:~
 : This function creates html5:h1, html5:h2, html5:h3, html5:h4, or html5:h5. It uses functions single:apply-corresp-node, single:apply-join and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $head: Defining kind of head-element
 :)

declare function single:h
    ( $node, $mode, $file, $head as xs:string )  {
    element {$head}
    {
      if ($node[@type = "sub"]) then
        attribute class {"sub-head"}
      else
        (),
      if ($node/@copyOf) then
        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
      else if ($node/@result) then
        single:apply-join($node, $mode, $file)
      else
        single:transform($node/node(), $mode, $file)
    }
};

(:~
 : This function creates a representation of a page beginning.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 :)

declare function single:pb( $node, $mode ) {
    element html5:span {
        attribute class {"pb"},
        concat
        (
          string("|" || string($mode)),
          string($node/@n),
          string("|")
        )
    }
};

(:~
 : This function creates a representation of tei:note: the supercript note-anchor with an accordion div.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $icon: The kind of icon
 : @param $class: class for the div which contains the content of tei:note
 :)

declare function single:note ( $node, $mode, $file, $class as xs:string ) {
    let $tempId := 'K' || util:uuid()
        return
        element html5:span
        {
            attribute class {"note"},
            element html5:a
            {
                attribute role {"button"},
                attribute data-toggle {"collapse"},
                attribute aria-expanded {"false"},
                attribute href {string("#" || $tempId)},
                attribute aria-controls {$tempId},
                element html5:sup
                {
                    data($node/tei:label[@type="note_anchor"])
                }
            }
            ,
            element html5:span
            {
                attribute class {"collapse normal-color"},
                attribute id {$tempId},
                element html5:div
                {
                    attribute class {$class},
                    single:transform($node/node(), $mode, $file)
                }
            }
        }
 };
 
 (:~
 : This function creates a representation of tei:note with @type="editorial"/"kmg": an icon linking to an accordion div.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $icon: The kind of icon
 : @param $class: class for the div which contains the content of tei:note
 :)

declare function single:note ( $node, $mode, $file, $icon as xs:string, $class as xs:string ) {
    let $tempId := 'K' || util:uuid()
        return
        element html5:span
        {
            attribute class {"note note-icon"},
            element html5:a
            {
                attribute role {"button"},
                attribute data-toggle {"collapse"},
                attribute aria-expanded {"false"},
                attribute href {string("#" || $tempId)},
                attribute aria-controls {$tempId},
                element html5:i
                {
                    attribute class {$icon}
                }
            },
            if (commons:starts-next-text-with-punctuation($node, $mode)) then
                ()
            else
                text{" "},
            element html5:span
            {
                attribute class {"collapse"},
                attribute id {$tempId},
                element html5:div
                {
                    attribute class {$class},
                    single:transform($node/node(), $mode, $file)
                }
            }
        }
 };

(:~
 : This function is differentiates between cases of @from and @to in <citedRange>
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 :)

declare function single:pages ( $node ) {
    if ($node/@from = $node/@to) then
        data($node/@from)
    else if (not( $node/@to)) then
        data($node/@from) || "ff"
    else
        data($node/@from) || "-" || data($node/@to)
};

 (:~
 : This function creates a representation of the attributes in <citedRange>
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 :)

declare function single:citedRange ( $node ) {
    if ($node/tei:citedRange/@unit = "page") then
        element html5:span
        {
            attribute class {"seitenangabe"},
            "S. " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else if ($node/tei:citedRange/@unit = "chapter") then
        element span
        {
            attribute class {"kapitelanagabe"},
            "Kapitel " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else if ($node/tei:citedRange/@unit = "column") then
        element span
        {
            attribute class {"Spaltenangabe"},
            "Sp. " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else if ($node/tei:citedRange/@unit = "paragraph") then
        element span
        {
            attribute class {"paragrafanagabe"},
            "§ " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else if ($node/tei:citedRange/@unit = "sheet") then
        element span
        {
            attribute class {"blattangabe"},
            "Blatt " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else if ($node/tei:citedRange/@unit = "vers") then
        element span
        {
            attribute class {"versanagabe"},
            "Vers " || single:pages ($node/tei:citedRange[1]) ||
            "."
        }
    else
    ()           
};

(:~
 : This function calls the element with the full-link.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 :)

declare function single:linkelement( $node ){
    $node/root()//*[@xml:id = data(replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', ''))][1]
};

(:~
 : This function calls the corresp/ref/target/resp-attribute of the element with the full-link. It calls the function single:linkelement.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:fullink( $node ){
    single:linkelement($node)/attribute()[name() = ('corresp', 'ref', 'target', 'resp')]
};

(:~
 : This function replaces the internal link with the full link and creates an icon that works as a link. It calls the function single:fulllink.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $abbrev: placeholder in the TEI-XML
 : @param $normalization: Base-URL of the actual linktarget
 : @param $suffix: Adding a suffix to the link
 : @param $icon: Defines the kind of icon
 :)

declare function single:fulllinkicon
    ( $node, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    element html5:a {
        attribute class {"text-link"},
        attribute href {replace(data(single:fullink($node)), $abbrev, $normalization) || $suffix},
        element html5:i {
            attribute class { $icon }
        },
                            element html5:span {
              $abbrev || " "|| $normalization
        }
    }
};

(:~
 : This function creates an icon that works as a link.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $abbrev: placeholder in the TEI-XML
 : @param $normalization: Base-URL of the actual linktarget
 : @param $suffix: Adding a suffix to the link
 : @param $icon: Defines the kind of icon
 :)

declare function single:linkicon
    ( $node, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    if ($abbrev = "zotero:") then
        single:linkzoteroPopover($node, $icon)
    else if ($abbrev = "gnd:") then
        single:linkPNDPover($node, $icon)
    else if ($abbrev = "dnb:") then
        single:linkDNBPover($node, $icon)
        (: be sure that we only replace lido links, thus the check for fa-image :)
    else if ($abbrev = "textgrid:" and $icon = "far fa-image") then
        single:linkArtPover($node, $icon)
    else (
        element html5:a {
            attribute class {"text-link"},
            attribute href {replace($node/attribute()[name() = ('corresp', 'ref', 'target')], $abbrev, $normalization) || $suffix},
            element html5:i {
                attribute class { $icon }
            }
        }
        )

};

(:~
 : Creates a zotero link popover. It is just a wrapper around the
 : generic function createPopover and sets the correct id by
 : replacing zotero: with just the id. It sets the title
 : to Literaturnachweis and the default content is Laden ...
 : until Javascript replaces the content
 :
 : @author Johannes Biermann
 : @version 1.0
 : 
 : @param $icon: Defines the kind of icon
 : @param $node: The current node
:)
declare function single:linkzoteroPopover($node, $icon as xs:string) {
    single:createPopover(
        $icon,
        'kmg-bib',
        'Literaturnachweis',
        'Laden ...',
        normalize-space(replace($node/attribute()[name() = ('corresp', 'ref', 'target')], "zotero:", "")),
        single:citedRange($node)
        )
};
(:~
 : Creates a GND Popover, content is loaded via kmg.js
 :
 : @author Johannes Biermann
 : @version 1.0
 : 
 : @param $icon: Defines the kind of icon
 : @param $node: The current node
:)
declare function single:linkPNDPover($node, $icon as xs:string) {
    single:createPopover(
        $icon,
        'kmg-pnd',
        'Angaben aus der GND',
        'Laden ...',
         normalize-space(replace($node/attribute()[name() = ('corresp', 'ref', 'target')], "gnd:", "")),
        ""
        )
};
(:~
 : Creates a DNB Popover, content is loaded via kmg.js
 :
 : @author Max Zeterberg
 : @version 1.0
 : 
 : @param $icon: Defines the kind of icon
 : @param $node: The current node
:)
declare function single:linkDNBPover($node, $icon as xs:string) {
    single:createPopover(
        $icon,
        'kmg-dnb',
        'Angaben aus der DNB',
        'Laden ...',
         normalize-space(replace($node/attribute()[name() = ('corresp', 'ref', 'target')], "dnb:", "")),
        ""
        )
};
(:~
 : Creates a LIDO Popover, content is loaded via kmg.js
 :
 : @author Johannes Biermann
 : @version 1.0
 : 
 : @param $icon: Defines the kind of icon
 : @param $node: The current node
:)
declare function single:linkArtPover($node, $icon as xs:string) {
    single:createPopover(
        $icon,
        'kmg-art',
        'Kunstwerk',
        'Laden ...',
         normalize-space(replace($node/attribute()[name() = ('corresp', 'ref', 'target')], "textgrid:", "")),
        ''
        )
};

(:~
 : This function creates a popover https://getbootstrap.com/docs/3.3/javascript/#popovers
 :  with a font-awesome icon
 : A popover is a thing where you can click to see some more information, another 
 : click somewhere else closes ist.
 : 
 : @author Johannes Biermann
 : @version 1.0
 : @param $icon: Defines the kind of icon
 : @param $class: the html class for this element type. Usually you use the class to bind
 : javascript actions for a popover. See kmg.js for an example
 : you can leave this empty for static popovers
 : @param $title: the title for the popover box. The box has two rows: title and content
 : @param $content: the content of the popover. Via Javascript you can replace this later
 : or leave it static
 : @param $id: an ID which will be set to data-id and can be accessed via Javascript
 : you can leave this empty for static popovers
 : @param $prefixText: usefull if additional data should be fetched via AJAX and combined
 : with data inside this transforamtion, see $('.kmg-bib').click in kmg.js as example
 :)
declare function single:createPopover($icon as xs:string, $class as xs:string, $title as xs:string, $content as xs:string, $id as xs:string, $prefixText) {
    element html5:a {
        attribute class {$class},
        attribute data-container {"body"},
        attribute data-content {$content},
        attribute data-html {"data-html"},
        attribute data-placement {"bottom"},
        attribute data-title {$title},
        attribute data-toggle {"popover"},
        attribute data-id {$id},
        attribute data-prefix {fn:string($prefixText)},
        attribute role {"button"},
        attribute tabindex {"-1"},
        attribute data-original-title {""},
        attribute title {""},
        element html5:i {
            attribute class { $icon }
        }, 
        (: this contains any additional data which should be shown as prefixText 
        e.g. rendering of data via AJAX and showing information from the XML 
        transcription, too.
        :)
        element html5:span {
            attribute style { "display:none;"},
             $prefixText
        }        
    }
};

(:~
 : This function creates a span around the text that was tagged as some kind of link and calls the functions single:apply-corresp-node, single:transform, single:createPopOver and single:fulllinkicon or single:linkicon.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining a certain class for the link
 :)

declare function single:spanicon
    ( $node, $mode, $file, $class as xs:string, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    (element html5:span {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file),
        string(" "),
        (: the target is different than what linkPNDPover defines :)
        if (starts-with($node/attribute()[name() = ('corresp', 'ref', 'target')], "#" )
            and ($abbrev = "gnd:" )) then
            single:createPopover(
                 $icon,
                'kmg-pnd',
                'Angaben aus der GND',
                'Laden ...',
            normalize-space(replace(data(single:fullink($node)), $abbrev, "")),
            ''        
        )
        else if (starts-with($node/attribute()[name() = ('corresp', 'ref', 'target')], "#" )
            and ($abbrev = "dnb:" )) then
            single:createPopover(
                 $icon,
                'kmg-dnb',
                'Angaben aus der DNB',
                'Laden ...',
            normalize-space(replace(data(single:fullink($node)), $abbrev, "")),
            ''        
        )
        else if (starts-with($node/attribute()[name() = ('corresp', 'ref', 'target')], "#" )) then
            single:fulllinkicon($node, $abbrev, $normalization, $suffix, $icon)
        else
            single:linkicon($node, $abbrev, $normalization, $suffix, $icon),
        if (commons:starts-next-text-with-punctuation($node, $mode)) then
            ()
        else
            text{" "}
        }
    )
};

(:~
 : This function replaces the abbreviation with the abbreviated link. It calls the functions single:spanicon and single:linkelement.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $class: Defining a certain class for the link
 :)

declare function single:abbrevtolink( $node, $mode, $file, $class as xs:string, $icon as xs:string ){
    if ($node[starts-with(@ref, "#")]) then
        if (single:linkelement($node)[contains(@ref, "gnd")]) then
            single:spanicon($node, $mode, $file, $class, "gnd:", "http://d-nb.info/gnd/", "", $icon)
        else if (single:linkelement($node)[contains(@ref, "dnb")]) then
            single:spanicon($node, $mode, $file, $class, "dnb:", "http://d-nb.info/", "", $icon)
        else if (single:linkelement($node)[contains(@ref, "textgrid")]) then
            single:spanicon($node, $mode, $file, $class, "textgrid:", "/exist/apps/sade/textgrid/data/kmg-lido/", ".xml", $icon)
        else
            ()
    else
        if ($node[contains(@ref, "gnd")]) then
            single:spanicon($node, $mode, $file, $class, "gnd:", "http://d-nb.info/gnd/", "", $icon)
        else if ($node[contains(@ref, "dnb")]) then
            single:spanicon($node, $mode, $file, $class, "dnb:", "http://d-nb.info/", "", $icon)
        else if ($node[contains(@ref, "textgrid")]) then
            single:spanicon($node, $mode, $file, $class, "textgrid:", "/exist/apps/sade/textgrid/data/kmg-lido/", ".xml", $icon)
        else
            ()
};

(:~
 : This function uses the functions single:apply-corresp-node and single:transform to transform tei:div.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:div( $node, $mode, $file ){
    element html5:div
    {
      if ($node/@copyOf) then
        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
      else if ($node/@result) then
        single:apply-join($node, $mode, $file)
      else
        single:transform($node/node(), $mode, $file)
    }  
};

(:~
 : This function uses the functions single:h, single:apply-corresp-node, single:apply-join, and single:transform to transform tei:div.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:head( $node, $mode, $file ){
  if ($node/parent::tei:figure) then
            ()
        else if ($node/parent::tei:table) then
            element html5:caption
            {
                if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                single:transform($node/node(), $mode, $file)
            }
        else
            (if ($node/parent::tei:div[@type = "part"]) then
                single:h($node, $mode, $file, "html5:h2" )
            else if ($node/parent::tei:div[@type = "chapter"]) then
                if ($node/ancestor::tei:text[@type = "commentary"]) then
                    ()
                else
                    single:h($node, $mode, $file, "html5:h3" )
            else if ($node/parent::tei:div[@type = "section"]) then
                single:h($node, $mode, $file, "html5:h4" )
            else
                single:h($node, $mode, $file, "html5:h5" )
            )  
};

(:~
 : This function uses the functions single:apply-corresp-node, single:apply-join, and single:transform to transform tei:item.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:listitem( $node, $mode, $file ){
    if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'label') then
      element html5:li
      {
        if ($node/@xml:id) then
          attribute id {$node/@xml:id}
        else
          (),
        attribute class {"labeled_item"},
        element html5:span
        {
            attribute class {"list-label"},
            data($node/preceding-sibling::tei:label[@type="list"][1])
        },
        if ($node/@copyOf) then
          single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
          single:apply-join($node, $mode, $file)
        else
          single:transform($node/node(), $mode, $file)
      }
  else if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'app') then
      element html5:li
      {
        if ($node/@xml:id) then
          attribute id {$node/@xml:id}
        else
          (),
        attribute class {"labeled_item"},
        element html5:span
        {
            attribute class {"list-label"},
            data($node/preceding-sibling::tei:app[1]/*[@wit="#" || $mode]/tei:label[@type="list"])
        },
        if ($node/@copyOf) then
          single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
          single:apply-join($node, $mode, $file)
        else
          single:transform($node/node(), $mode, $file)
      }
  else if ($node/parent::tei:*/parent::tei:app/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'app') then
      element html5:li
      {
        if ($node/@xml:id) then
          attribute id {$node/@xml:id}
        else
          (),
        attribute class {"labeled_item"},
        element html5:span
        {
            attribute class {"list-label"},
            data($node/parent::tei:*/parent::tei:app/preceding-sibling::tei:app[1]/*[@wit="#" || $mode]/tei:label[@type="list"])
        },
        if ($node/@copyOf) then
          single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
          single:apply-join($node, $mode, $file)
        else
          single:transform($node/node(), $mode, $file)
      }
  else
      element html5:li
      {
        if ($node/@xml:id) then
          attribute id {$node/@xml:id}
        else
          (),
        attribute class {"unlabeled_item"},
        if ($node/@copyOf) then
          single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
        else if ($node/@result) then
          single:apply-join($node, $mode, $file)
        else
          single:transform($node/node(), $mode, $file)
      }
};

(:~
 : This function uses the function single:blockelement to transform tei:p.
 : Since we have xhtml:div nested in p, e.g. for compositor's errors, we need to choose xhtml:div here instead of xhtml:p. Otherwise the HTML content model is violated.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:paragraph( $node, $mode, $file ){
    if ($node/ancestor::tei:figure) then
        element html5:figcaption
        {
           single:transform($node/node(), $mode, $file) 
        }
    else if ($node/parent::tei:item/ancestor::tei:list) then
        single:blockelement($node, $mode, $file, "p listp")
    else
        single:blockelement($node, $mode, $file, "p")  
};

declare function single:q-quote( $node, $mode, $file, $class as xs:string ){
    if ($node/@rend = "none") then
        single:qqmclass( $node, $mode, $file, "", "", $class )
    else if ($node[contains(@rend, "double")]) then
        if ($node/ancestor::tei:text[@type = "commentary"]) then
            single:qqmclass( $node, $mode, $file, "&#x201E;", "&#x201C;", $class )
        else if ($node[contains(@rend, "qm_low")]) then
            single:qqmclass( $node, $mode, $file, "&#x201E;", "&#x201C;", $class )
        else if ($node[contains(@rend, "qm_high")]) then
            single:qqmclass( $node, $mode, $file, "&#x201C;", "&#x201D;", $class )
        else if ($node[contains(@rend, "g_out")]) then
            single:qqmclass( $node, $mode, $file, "&#xAB;", "&#xBB;", $class )
        else if ($node[contains(@rend, "g_in")]) then
            single:qqmclass( $node, $mode, $file, "&#xBB;", "&#xAB;", $class )
        else
            (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                single:qqmclass( $node, $mode, $file, "&#x201E;", "&#x201C;", $class )
            else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                single:qqmclass( $node, $mode, $file, "&#x201C;", "&#x201D;", $class )
            else if (contains(single:wit-rend($node, $mode), "g_out")) then
                single:qqmclass( $node, $mode, $file, "&#xAB;", "&#xBB;", $class )
            else if (contains(single:wit-rend($node, $mode), "g_in")) then
                single:qqmclass( $node, $mode, $file, "&#xBB;", "&#xAB;", $class )
            else
                ()
            )
    else if ($node[contains(@rend, "single")]) then
        if ($node/ancestor::tei:text[@type = "commentary"]) then
            single:qqmclass( $node, $mode, $file, "&#x201A;", "&#x2018;", $class )
        else if ($node[contains(@rend, "qm_low")]) then
            single:qqmclass( $node, $mode, $file, "&#x201A;", "&#x2018;", $class )
        else if ($node[contains(@rend, "qm_high")]) then
            single:qqmclass( $node, $mode, $file, "&#x2018;", "&#x2019;", $class )
        else if ($node[contains(@rend, "g_out")]) then
            single:qqmclass( $node, $mode, $file, "&#x2039;", "&#x203A;", $class )
        else if ($node[contains(@rend, "g_in")]) then
            single:qqmclass( $node, $mode, $file, "&#x203A;", "&#x2039;", $class )
        else
            (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                single:qqmclass( $node, $mode, $file, "&#x201A;", "&#x2018;", $class )
            else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                single:qqmclass( $node, $mode, $file, "&#x2018;", "&#x2019;", $class )
            else if (contains(single:wit-rend($node, $mode), "g_out")) then
                single:qqmclass( $node, $mode, $file, "&#x2039;", "&#x203A;", $class )
            else if (contains(single:wit-rend($node, $mode), "g_in")) then
                single:qqmclass( $node, $mode, $file, "&#x203A;", "&#x2039;", $class )
            else
                ()
            )
    else
        ()
};

declare function single:transform($nodes as node()*, $mode as xs:string*,
$file as xs:string) as node()*
{
  for $node in $nodes
  return
    (
      if ($node/@xml:id) then
        commons:make-text-anchor(string($node/@xml:id)
        (: Hier werden alle Elemente mit xml:id mit html-Ankerpunkten versehen :)
    )
    else
      (),
    typeswitch($node)
    case text() return
      let $text := text{replace($node, "@", " ")}
      return
         $text

    case comment() return
      ()

    case element(tei:app) return
      (if ($node/child::*[matches(@wit, $mode)]) then
        single:transform($node/child::*[matches(@wit, $mode)], $mode, $file)
      else
        single:transform($node/child::tei:lem, $mode, $file),
      if (single:is-next-text-in-app($node)
      and single:is-next-app-omitted($node, $mode)) then
        ()
      else
        commons:make-whitespace-after($node))

    case element(tei:back) return
      single:transform($node/node(), $mode, $file)

(:~
 : This transformation uses the function single:spanicon to create Link-Icons for tei:bibl with different classes for @type="KMG/commentary". It creates a whitespace if the tei:bibl follows directly a tei:quote or tei:q.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)
    
    case element(tei:bibl) return
        if ($node/@type = "KMG" or $node/@type = "commentary") then
            if ($node/preceding-sibling::tei:quote and $node/parent::tei:cit) then
                (element html5:span
                {
                    " "
                },
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book kmg")
                )
            else
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book kmg")
        else
            if ($node/preceding-sibling::tei:quote and $node/parent::tei:cit) then
                (element html5:span
                {
                    " "
                },
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
                )
            else
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")

    case element(tei:cb) return
      element html5:span
      {
        attribute class {"cb"},
        string("|" || replace($node/@edRef, '[# ]+', '') || $node/@n || "|")
      }

    case element(tei:cell) return
        if ($node/parent::tei:row/parent::tei:table/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/parent::tei:row/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/@rend) then
            element html5:td
            {
               attribute class {data($node/@rend)},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else
            element html5:td
            {
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }

    case element(tei:choice) return
        if ($node/tei:sic) then
            element html5:span
            {
                attribute class {"resp-tt"},
                if (not($node/tei:sic/node())) then
                    element html5:span {
                        attribute class { "kmg" },
                        ""
                    }
                else
                    single:spanclass($node/tei:sic, $mode, $file, "kmg" ),
                element html5:span
                {
                    attribute class {"resp-tttext"},
                    if (not($node/tei:corr/node())) then
                        element html5:span {
                            attribute class { "kmg" },
                            "Ø"
                        }
                    else
                        element html5:span {
                            attribute class { "kmg" },
                            single:transform($node/tei:corr/node(), $mode, $file)
                        },
                    " ",
                    if (single:fullink( $node/tei:corr )) then
                        ("[" || normalize-space(data(single:linkelement($node/tei:corr))) || " ",
                        single:createPopover(
                             "fas fa-user",
                            'kmg-pnd',
                            'Angaben aus der GND',
                            'Laden ...',
                         normalize-space(replace(data(single:fullink($node/tei:corr)), "gnd:", "")),
                         ''),
                         "]")
                    else
                        "[" || normalize-space(data(single:linkelement($node/tei:corr))) || "]"
                }
            }
        else if ($node/tei:abbr) then
            if ($node/tei:abbr/@copyOf) then
                single:apply-corresp-node($node, $node/tei:abbr/@copyOf, $mode, $file, false())
            else
                single:transform($node/tei:abbr/node(), $mode, $file)
        else
            ()

    case element(tei:div) return
        if ($node/@type = "cover") then
            single:divclass($node, $mode, $file, "row")
        else
            single:div($node, $mode, $file)

    case element(tei:figDesc) return
        ()
    
    case element(tei:figure) return
        if ($node/@type = "cover") then
            element html5:div
            {
                if ($node/parent::tei:*/child::tei:figure[4]) then
                    attribute class {"col-md-3"}
                else if ($node/parent::tei:*/child::tei:figure[3]) then
                    attribute class {"col-md-4"}
                else if ($node/parent::tei:*/child::tei:figure[2]) then
                    attribute class {"col-md-6"}
                else
                    attribute class {"col-md-9"},
                element html5:figure
                {
                    attribute class {"thumbnail"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                } 
            }
        else
            element html5:figure
            {
                attribute class {"image"},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }

    case element (tei:floatingText) return
        single:divclass($node, $mode, $file, "floating-text")

    case element(tei:foreign) return
      if ($node[@xml:lang = "el"]) then
          single:spanclass( $node, $mode, $file, "hi-greek" )
      else
          single:spanclass( $node, $mode, $file, "foreign" )

    case element(tei:front) return
      element html5:div
      {
        attribute class {"preface"},
        single:transform($node/node(), $mode, $file)
      }

    case element(tei:graphic) return
        element html5:a
        {
            attribute href {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
            element html5:img
            {
                attribute src {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
                attribute alt {data(replace(string($node/ancestor::tei:figure/descendant::tei:figDesc), "@", " "))},
                if ($node/parent::tei:figure[@type = "cover"]) then
                    attribute class {"cover"}
                else    
                    (),
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        }

    case element(tei:head) return
        if ($node[@type="ToC"]) then
            ()
        else
            single:head($node, $mode, $file)

    case element(tei:hi) return
      if ($node/ancestor::tei:text[@type = "commentary"]) then
          single:spanclass( $node, $mode, $file, "italics" )
      else if ($node/@rend) then
          if ($node[@rend = "spaced"] and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
              single:spanclass( $node, $mode, $file, "spaced-out spaced-out-no-right-padding" )
          else
              single:spanclass( $node, $mode, $file, $node/@rend )
      else
        (if (contains(single:wit-rend($node, $mode), "body_type")) then
            single:spanclass( $node, $mode, $file, "body_type" )
        else if (contains(single:wit-rend($node, $mode), "bold")) then
            single:spanclass( $node, $mode, $file, "bold" )
        else if (contains(single:wit-rend($node, $mode), "italics")) then
            single:spanclass( $node, $mode, $file, "italics" )
        else if (contains(single:wit-rend($node, $mode), "small_caps")) then
            single:spanclass( $node, $mode, $file, "small_caps" )
        else if (contains(single:wit-rend($node, $mode), "spaced") and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
            single:spanclass( $node, $mode, $file, "spaced spaced-out-no-right-padding" )
        else if (contains(single:wit-rend($node, $mode), "spaced")) then
            single:spanclass( $node, $mode, $file, "spaced" )
        else if (contains(single:wit-rend($node, $mode), "under")) then
            single:spanclass( $node, $mode, $file, "under" )
        else if (contains(single:wit-rend($node, $mode), "versal")) then
            single:spanclass( $node, $mode, $file, "versal" )
        else
            single:spanclass( $node, $mode, $file, "hi" )
        )

    case element(tei:item) return
      if ($node/parent::tei:figure) then
            ()
      else
          single:listitem($node, $mode, $file)

    (: Sobald neue Werte für @result erlaubt sind, muss dieser Teil angepasst werden :)
    case element(tei:join) return
        switch($node/@result)
        case "div" return
            single:div($node, $mode, $file)
        case "head" return
            single:head($node, $mode, $file)
        case "item" return
          single:listitem($node, $mode, $file)
        case "p" return
            single:paragraph($node, $mode, $file)
        case "q" return
            single:q-quote($node, $mode, $file, "")
        case "quote" return
            single:q-quote($node, $mode, $file, "quote")
        (: the default case is that join has no @result :)
        default return
            single:apply-join($node, $mode, $file)

    case element(tei:l) return
        if (not($node/following-sibling::tei:l)) then
            single:spanclass( $node, $mode, $file, "line" )
        else
            (single:spanclass( $node, $mode, $file, "line" ),
            element br{})

    case element(tei:label) return
      if ($node[@type = "note_anchor"]) then
          (if ($node/parent::*[@anchored = "false"]) then
              element html5:span
                {
                  attribute class {"anchored_false"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
            else
                element html5:span
                {
                  attribute class {"note_anchor"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
          )
      else if ($node[@type = "list"]) then
          ()
      else if ($node[@type = "head"]) then
          single:spanclass( $node, $mode, $file, "headlabel" ) 
      else
          single:spanclass( $node, $mode, $file, "label-wrapper" )

    case element(tei:lb) return
      if ($node/preceding-sibling::*[self::tei:titlePart or self::tei:byline
      or self::tei:docImprint or self::tei:head]) then
        ()
      else
        element html5:br{}

    case element(tei:lg) return
        single:blockelement($node, $mode, $file, "linegroup")

    case element(tei:list) return
        if ($node/@type = ("unspecified", "index")) then
            element html5:ul
              {
                (if ($node/parent::tei:item) then
                  attribute class {"nested-list unlabeled_list" ||
                    (if ($node/@rend= "blockquote") then " blockquote"
                    else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                    else if ($node/@rend = "small_letters") then " small-letters"
                    else "")}
                else if ($node/ancestor::tei:rdg) then
                  attribute class {"rdg-list unlabeled_list" ||
                    (if ($node/@rend= "blockquote") then " blockquote"
                    else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                    else if ($node/@rend = "small_letters") then " small-letters"
                    else "")}
                else
                  attribute class {"text-list unlabeled_list" ||
                    (if ($node/@rend= "blockquote") then " blockquote"
                    else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                    else if ($node/@rend = "small_letters") then " small-letters"
                    else "")}),
                single:transform($node/node(), $mode, $file)
              }
        else if ($node/@type = "abbildungstext") then
            ()
        else
          element html5:ul
          {
            (if ($node/parent::tei:item) then
              attribute class {"nested-list labeled_list" ||
              (if ($node/@rend= "blockquote") then " blockquote"
                else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                else if ($node/@rend = "small_letters") then " small-letters"
                else "")}
            else if ($node/ancestor::tei:rdg) then
              attribute class {"rdg-list labeled_list" || 
                (if ($node/@rend= "blockquote") then " blockquote"
                else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                else if ($node/@rend = "small_letters") then " small-letters"
                else "")}
            else
              attribute class {"text-list labeled_list" || 
                (if ($node/@rend= "blockquote") then " blockquote"
                else if ($node/@rend= "blockquoteitalics") then " blockquote italics"
                else if ($node/@rend = "small_letters") then " small-letters"
                else "")}),
            single:transform($node/node(), $mode, $file)
          }

    case element(tei:milestone) return
        if ($node/@unit = "page"
        and matches($node/@edRef, $mode)) then
            single:pb($node, $mode)
        else if ($node/@type = "paragraph") then
            element html5:br{}
        else
            ()

(:~
 : This transformation uses the function single:abbrevtolink to create Link-Icons for the different tei:name-elements. If the tei:name refers to another tei:name-element in the same document, the internal link will be replaced by the external link of the other tei:element. If the tei-element does not have a ref-attribute, no link will created.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

    case element(tei:name) return
        if ($node/@type = "art") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "artname")
            else
                single:abbrevtolink($node, $mode, $file, "artname", "far fa-image")
        else if ($node/@type = "person") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "persname")
            else
                single:abbrevtolink($node, $mode, $file, "persname", "fas fa-user")
        else if ($node/@type = "org") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "orgname")
            else
                single:abbrevtolink($node, $mode, $file, "orgname", "fas fa-external-link-alt")
        else if ($node/@type = "music") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "musicname")
            else
                single:abbrevtolink($node, $mode, $file, "musicname", "fas fa-music")
        else if ($node/@type = "periodica") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "periodname")
            else
                single:abbrevtolink($node, $mode, $file, "periodname", "fas fa-newspaper")
        else if ($node/@type = "brand") then
            if (not($node/@ref)) then
                single:spanclass($node, $mode, $file, "brandname")
            else
                single:abbrevtolink($node, $mode, $file, "brandname", "far fa-copyright")
        else
            ()

    case element(tei:note) return
        if ($node/@type = "KMG") then
            single:note ($node, $mode, $file, "fas fa-comment-alt", "well kmg")
        else if ($node/@type = "editorial") then
            single:note ($node, $mode, $file, "well editorial")
        else            
            single:note ($node, $mode, $file, "well")

    case element(tei:p) return
        single:paragraph($node, $mode, $file)

    case element(tei:pb) return
      if (matches($node/@edRef, $mode)) then
        single:pb($node, $mode)
      else if (not($node/following-sibling::*[1][self::tei:pb])) then
        commons:make-whitespace-after($node)
      else
        ()

    case element(tei:q) return
        single:q-quote($node, $mode, $file, "")

    case element(tei:quote) return
        single:q-quote($node, $mode, $file, "quote")

    case element(tei:rdg) return
        single:transform($node/node(), $mode, $file)

    case element(tei:ref) return
        element html5:span {
            attribute class { "ref" },
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file),
            string(" "),
            if (starts-with($node/@target, "#" )) then
                element html5:a {
                    attribute class {"text-link"},
                    attribute href {$node/@target},
                    element html5:i {
                        attribute class { "fas fa-link" }
                    }
                }
            else
                element html5:a {
                    attribute class {"text-link"},
                    attribute href {replace($node/@target, "textgrid:", "/exist/apps/sade/textgrid/data/kmg/")},
                    element html5:i {
                        attribute class { "fas fa-link" }
                    }
                },
            if (commons:starts-next-text-with-punctuation($node, $mode)) then
                ()
            else
                text{" "}
        }

    case element(tei:row) return
        element html5:tr
        {
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }

    (: Hier könnten noch die verschiedenen Anwendungsfälle (choice, copy, eyecatcher, same, transposition ) spezifiziert werden :)
    case element(tei:seg) return
      if ($node/@copyOf) then
        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
      else if ($node/@type = "margin") then
            element html5:span
            {
                attribute class {"margin"},
                text{"{"},
                single:transform($node/node(), $mode, $file),
                text{"}"}
            }
      else if ($node/@type = "quote_add") then
          single:spanclasstooltip($node, $mode, $file, "")
      else if ($node/@type = "quote_hi") then
          if ($node/@rend) then
              if ($node[@rend = "spaced"] and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
                  single:spanclasstooltip($node, $mode, $file, "spaced-out spaced-out-no-right-padding")
              else
                  single:spanclasstooltip($node, $mode, $file, $node/@rend)
         else
             if (contains(single:wit-rend($node, $mode), "body_type")) then
                single:spanclasstooltip($node, $mode, $file, "body_type")
             else if (contains(single:wit-rend($node, $mode), "bold")) then
                single:spanclasstooltip($node, $mode, $file, "bold")
             else if (contains(single:wit-rend($node, $mode), "italics")) then
                single:spanclasstooltip($node, $mode, $file, "italics")
             else if (contains(single:wit-rend($node, $mode), "small_caps")) then
                single:spanclasstooltip($node, $mode, $file, "small_caps")
             else if (contains(single:wit-rend($node, $mode), "spaced") and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
                single:spanclasstooltip($node, $mode, $file, "spaced spaced-out-no-right-padding")
             else if (contains(single:wit-rend($node, $mode), "spaced")) then
                single:spanclasstooltip($node, $mode, $file, "spaced")
             else if (contains(single:wit-rend($node, $mode), "under")) then
                single:spanclasstooltip($node, $mode, $file, "under")
             else if (contains(single:wit-rend($node, $mode), "versal")) then
                single:spanclasstooltip($node, $mode, $file, "versal")
             else
                single:spanclasstooltip($node, $mode, $file, "hi")
      else
         single:transform($node/node(), $mode, $file)

    case element (tei:signed) return
        single:divclass($node, $mode, $file, "signed")

    case element(tei:table) return
        if ($node/tei:trailer) then
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               },
               element html5:figcaption
               {
                   if ($node/tei:trailer/@copyOf) then
                        single:apply-corresp-node($node/tei:trailer, $node/tei:trailer/@copyOf, $mode, $file, false())
                    else
                        single:transform($node/tei:trailer/node(), $mode, $file) 
               }
            }
        else
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               }
            }

(:~
 : This transformation creates a representation of the tei:Header. It uses the function single:spanicon to create Link-Icons for tei:name (tei:author, tei:editor, tei:respStmt) and single:linkicon for tei:bibl (in tei:witness).
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

    case element(tei:teiHeader) return
        element html5:div
        {
            attribute class {"header kmg"},
            element html5:h1
            {
                single:transform($node/tei:fileDesc/tei:titleStmt/tei:title[@type="main"]/node(), $mode, $file)
            },
            if ($node/tei:fileDesc/tei:titleStmt/tei:title[@type="sub"]) then
                (element html5:h1
                {
                    attribute class {"sub-head"},
                    single:transform($node/tei:fileDesc/tei:titleStmt/tei:title[@type="sub"]/node(), $mode, $file)
                }
                )
            else
                (),
            element html5:p
            {
                attribute class {"headerperson"},
                "Autor: ",
                single:spanicon($node/tei:fileDesc/tei:titleStmt/tei:author/tei:name[1], $mode, $file, "persname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
            },
            if ($node/tei:fileDesc/tei:titleStmt/tei:respStmt) then
                (for $other in $node/tei:fileDesc/tei:titleStmt/tei:respStmt/tei:name[1]
                return
                    element html5:p
                {
                    attribute class {"headerperson"},
                    $other/preceding-sibling::tei:resp/text() || ": ",
                    single:spanicon($other, $mode, $file, "persname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
                })
            else
                (),
            element html5:h5
            {
                attribute class {"sub-head"},
                "Bearbeiter*innen:"
            },
            element html5:ul
            {
                for $editor in $node/tei:fileDesc/tei:titleStmt/tei:editor/tei:name[1]
                let $fr := (" ", "_")
                let $to := (", ", " ")
                return
                    element html5:li
                    {
                        if (single:fullink( $editor)) then
                            single:spanicon($editor, $mode, $file, "persname headerperson", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
                        else
                            single:transform($editor/node(), $mode, $file),
                        " (" || string(functx:replace-multi($editor/parent::tei:editor/@role, $fr, $to)) || ")"
                    }
            },
            if (single:witness( $node/tei:fileDesc/tei:sourceDesc, $mode )/descendant::tei:witness[2]) then
                element html5:h5
                {
                    attribute class {"sub-head"},
                    "Dieser Textfassung zugrundeliegende Publikationen:"
                }
            else
                element html5:h5
                {
                    attribute class {"sub-head"},
                    "Dieser Textfassung zugrundeliegende Publikation:"
                },
            element html5:ul
            {
                for $publikation in single:witness( $node/tei:fileDesc/tei:sourceDesc, $mode )/descendant::tei:witness/tei:bibl
                    return
                        element html5:li
                        {
                            $publikation/preceding-sibling::tei:desc/text() || " ",
                            single:linkicon($publikation, "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
                        }
            }
        }

    case element(tei:text) return
        if ($node/@type = "commentary") then
            let $tempId1 := 'L' || util:uuid()
            let $tempId2 := 'M' || util:uuid()
                return
            element html5:div
            {
                attribute id {$tempId1},
                attribute class {"kmg"},
                element html5:div
                {
                    attribute class {"card"},
                    element html5:div
                    {
                        attribute class {"card-header"},
                        attribute id {"headingOne"},
                        element html5:h2
                        {                       
                            attribute class {"mb-0"},
                            element html5:button
                            {
                                attribute class {"btn btn-link"},
                                attribute data-toggle {"collapse"},
                                attribute aria-expanded {"true"},
                                attribute data-target {string("#" || $tempId2)},
                                attribute aria-controls {$tempId2},      
                                single:transform($node/descendant::tei:div[@type="chapter"]/tei:head[1]/node(), $mode, $file)
                            }
                            
                         }
                    },
                    element html5:div
                    {
                        attribute class {"collapse"},
                        attribute id {$tempId2},
                        attribute aria-labelledby {"headingOne"},
                        attribute data-parent {string("#" || $tempId1)},
                        element html5:div
                        {
                            attribute class {"card-body"},
                            single:transform($node/descendant::tei:div[@type="chapter"]/node(), $mode, $file)
                        }
                    }
                }
            }
        else
            single:transform($node/node(), $mode, $file)
    
    case element(tei:trailer) return
        ()

    default return
      single:transform($node/node(), $mode, $file),
      commons:make-whitespace-after($node)
  )
};


(:~
 : Finds and applies the node that is referenced with @copyOf.
 : In case of tei:join we have to apply only the child nodes of the corresponding
 : nodes because tei:join defines a new resulting element in @result that makes
 : the original nodes obsolete.
 : Changes from version 1.0 to 2.0: Instead of single:transform($corresp-node, $mode, $file) and single:transform($child, $mode, $file) it is now single:transform($corresp-node/node(), $mode, $file) and single:transform($child/node(), $mode, $file).
 :
 : @author Michelle Weidling
 : @author Max Zeterberg
 : @version 2.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @param $mode The current edition
 : @param $file
 : @param $join A flag indicating if the calling element is a tei:join
 : @return The transformed node the @copyOf refers to
 :)
declare function single:apply-corresp-node($node as node()*, $copyOf as xs:string, $mode as
xs:string, $file as xs:string, $apply-children as xs:boolean)
as node()*
{
  let $copyOf := replace($copyOf, "#", "")
  let $corresp-node := single:find-corresp-node($node, $copyOf)
  return
      if ($apply-children) then
          for $child in $corresp-node return
              single:transform($child/node(), $mode, $file)
      else
            single:transform($corresp-node/node(), $mode, $file)
};


(:~
 : Retrieves the target node of a given @copyOf-reference. 
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @return The node the @copyOf refers to
 :)
declare function single:find-corresp-node($node as node()*, $copyOf as xs:string) as node()?{
    $node/root()//*[@xml:id = $copyOf]
};


(:~
 : Creates the icon for a compositor's error and initiates the construction of
 : the respective modal div.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $mode The current edition
 : @return The modal div, a whitespace and the icon
 :)
declare function single:make-typo-corr($node as element(tei:choice),
$mode as xs:string, $file as xs:string) as node()*
{
  let $random-id := single:make-random-id()
  let $id := util:hash($random-id, "md5")
  let $edition :=
    if ($node/parent::tei:lem[not(@wit)]) then
      (: single file names don't have a hyphen, only the ones that are the
      result of semantic splitting :)
      if (matches($file, "-")) then
        let $author := substring-before($file, "-")
        let $full-file := doc($config:full-xmls || $author || "-full.xml")
        return $full-file//tei:witness[@n = "base-text"]/@xml:id/string()
      else
        $node/ancestor::tei:TEI//tei:witness[@n = "base-text"]/@xml:id
    else
      $mode
  return
    (single:make-typo-corr-modal($node, $edition, $mode, $id, $file),
    text{" "},
    <a class="comp-error" data-target="#{$id}">
      <i class="fa fa-wrench" aria-hidden="true" data-toggle="modal" />
    </a>)
};


(:~
 : Creates the modal div for a compositor's error.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $edition The edition in which an error occurs
 : @param $mode The current edition
 : @param $id The target id
 : @return The modal div
 :)
declare function single:make-typo-corr-modal($node as element(tei:choice),
$edition as xs:string, $mode as xs:string, $id as xs:string, $file as xs:string)
as element(html5:div)
{
  <div class="modal fade" role="dialog" id="{$id}">
    <div class="modal-dialog modal-typo">
      <div class="modal-content">
        <div class="modal-header">
          <button type="button" class="close" data-dismiss="modal">
            × <!-- times -->
          </button>
          {
            if ($node/tei:corr/@type = "authorial") then
              let $sic :=
                if ($node/tei:sic[not(child::node() or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Autorschaftliche Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
            else
              let $sic :=
                if ($node/tei:sic[not(child::* or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Editorische Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
          }
        </div>
      </div>
    </div>
  </div>
};


declare function single:is-next-text-in-app($node as node()*) as xs:boolean
{
  if ($node/following::text()[1][normalize-space(.) != ""][ancestor::tei:app])then
    true()
  else
    false()
};

declare function single:is-next-app-omitted($node as node()*, $mode as xs:string)
as xs:boolean
{
  let $next-app := $node/following-sibling::*[1][self::tei:app]
  return
    if ($next-app/child::tei:rdg[matches(@wit, $mode) and @type = "om"]) then
      true()
    else
      false()
};

(:~
 : In cases like tei:join several references are stated in a @target. This
 : function returns all corresponding nodes.
 : For KMG: functionality to create a whitespace before the first target has been replaced by a whitespace after each target.
 :
 : @author Michelle Weidling
 : @author Max-Ferdinand Zeterberg
 : @version 2.0
 : @param $target The @target of a e.g. tei:join
 : @param $mode The current edition
 : @return The nodes the @target refers to
 :)
declare function single:apply-join($node as element(tei:join), $mode as xs:string,
$file as xs:string) as node()*
{
    let $apply-only-children :=
        if ($node/@scope = "root") then
            false()
        else
            true()
    return
        for $copyOf in tokenize($node/@target, " ")
            let $applied-node := single:apply-corresp-node($node, $copyOf, $mode, $file, $apply-only-children)
(:            let $first-text := $applied-node[1]/descendant-or-self::text()[1]:)
            let $conjunction := text{" "}
(:              if (starts-with($first-text, ",")) then:)
(:                text{""}:)
(:              else:)
(:                text{""}:)
            return ($applied-node, $conjunction)
};

(:~
 : Creates a random ID of 40 letters consisting of Basic Latin letters and
 : numbers. This function is needed since the usage of generate-id() (which we
 : originally used for the linking between icons and a modal dialogue) lead to
 : ID duplettes.
 :)
declare function single:make-random-id() as xs:string
{
    let $charset := "ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz0123456789"
    let $id :=
        for $iii in 1 to 40 return
            let $number := util:random(40)
            return
                substring($charset, $number, 1)
    return string-join($id, "")
};
